#!/usr/bin/env python3


from utilities import draw_grid_canvas, start_timer

import sys
import random
import drawcanvas
import tkinter

from byugrid import Grid


def random_placement(grid):
    """
    Set 30% of the grid cells to random letters from the word 'cougars'
    :param grid: a grid
    :return: the same grid, but with random letters in it
    >>> grid1 = Grid.build([[None, None, None], [None, None, None], [None, None, None]])
    >>> random.seed(108)
    >>> random_placement(grid1)
    [['r', None, 'u'], ['r', None, None], [None, None, 'o']]
    """
    for x in range(grid.width):
        for y in range(grid.height):
            chance = random.randrange(10)
            if chance <= 2:
                letter = random.choice('cougars')
                grid.set(x, y, letter)
    return grid

    pass


def random_movement(grid):
    """
    Move every letter to a random neighboring cell. If a chosen cell is out of
    bounds, the letter disappears. If a chosen cell already has a letter in it,
    the new letter overwrites the old one.
    >>> grid1 = Grid.build([[None, None, None], ['a', None, 'c'], ['b', None, None], [None, None, None]])
    >>> random.seed(112)
    >>> random_movement(grid1)
    [[None, None, None], [None, None, None], ['a', 'b', 'c'], [None, None, None]]
    >>> grid1 = Grid.build([[None, None, None], ['a', None, 'c'], ['b', None, None], [None, None, None]])
    >>> random.seed(243)
    >>> random_movement(grid1)
    [[None, None, None], [None, 'c', None], [None, None, None], ['b', None, None]]
    >>> grid1 = Grid.build([[None, None, None], ['a', None, 'c'], ['b', None, None], [None, None, None]])
    >>> random.seed(333)
    >>> random_movement(grid1)
    [[None, None, 'c'], [None, 'a', None], [None, 'b', None], [None, None, None]]
    """
    new_grid = Grid(grid.width, grid.height)
    for x in range(grid.width):
        for y in range(grid.height):
            value = grid.get(x, y)
            new_value = new_grid.get(x, y)
            if value is not None:
                direction = random.randrange(4)
                # 0 means left
                if direction == 0 and grid.in_bounds(x - 1, y):
                    new_grid.set(x - 1, y, value)
                # 1 means up
                if direction == 1 and grid.in_bounds(x, y - 1):
                    new_grid.set(x, y - 1, value)
                # 2 means right
                if direction == 2 and grid.in_bounds(x + 1, y):
                    new_grid.set(x + 1, y, value)
                # 3 means down
                if direction == 3 and grid.in_bounds(x, y + 1):
                    new_grid.set(x, y + 1, value)
    for x in range(new_grid.width):
        for y in range(new_grid.height):
            value = new_grid.get(x, y)
            grid.set(x, y, value)
    return grid


# ************* Utility Functions Below here

def movie_action(grid, canvas, scale):
    """Do one round of the move, call in timer."""
    random_movement(grid)
    draw_grid_canvas(grid, canvas, scale)


def main():
    import argparse

    parser = argparse.ArgumentParser(description="Random letters")
    parser.add_argument('--width', type=int, default=30, help='width of the board, 30 by default')
    parser.add_argument('--height', type=int, default=30, help='height of the board, 30 by default')
    parser.add_argument('--speed', type=int, default=500, help='speed of each generation, 500ms by default')
    parser.add_argument('--scale', type=int, default=20, help='scale of board, 10 by default')
    args = parser.parse_args()

    # create the grid
    grid = Grid(args.width, args.height)
    grid = random_placement(grid)

    # draw the canvas
    canvas = drawcanvas.make_canvas(args.width * args.scale, args.height * args.scale, 'Randomize')
    draw_grid_canvas(grid, canvas, args.scale)

    # start a timer that will call movie_action every 30ms
    start_timer(canvas, 300, lambda: movie_action(grid, canvas, args.scale))

    # run the animation
    tkinter.mainloop()


if __name__ == '__main__':
    main()
